;########################################################################################
;########################################################################################
;
;	Projekt:		senso.asm							
;
;	Version:		1.0								
;
;	Bearbeiter: 		F. Schubert							
;
;	Datum:			14/10/2002
;
;	Funktion:		Geschicklichkeitsspiel "Senso" (nur EIN Timer!!)
;
;	nderungen:										
;				
;
;	Prozessor:		PIC16F876
;											
;########################################################################################
;########################################################################################



		list	p=pic16f876, f=inhx8m, n=85, st=off
						; Gltiger Prozessor = PIC16F876
		include <p16f876.inc>

		__config _XT_OSC & _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _CP_OFF & _LVP_OFF & _DEBUG_OFF

					;XT-Oscillator 3,6864MHz
					;Power-On aktiv, Brown-Out-Reset inaktiv
					;Watchdog inaktiv und Codeprotection inaktiv
					;LowVoltage_Programming inaktiv, DebugMode inaktiv


		errorlevel 0,-302,-305



;########################################################################################
;	Konstanten
;########################################################################################

;----------- Initialisierung -----------
TIMER0_RELOAD		equ	0xA4		; Reloadwert 256-92=164 fr Timer0
AUSSCHALTZEIT		equ	0x78		; nach 120s Sekunden wird abgeschaltet
;---------------------------------------

;------------ Betriebsmode -------------
START_UP		equ	0x00		; Ist Taster4 gedrckt? Dann Selbsthaltung!
ANIMATION		equ	0x01		; Anfangsanimation
CHOOSE_LEVEL		equ	0x02		; Schwierigkeitsgrad whlen
SEQUENCE_PROCESS	equ	0x03		; Die Ton/Farb-Sequenz wird vom Controller ausgegeben
WAIT_OUTPUT		equ	0x04		; Pause machen zwischen den einzelnen Farben
USER_PROCESS		equ	0x05		; Der User muss die Signal-Sequenz in der richtigen Reihenfolge wiederholen
U_SUCCESS		equ	0x06		; Der User hat die gesamte Signal-Sequenz geschafft
U_ERROR			equ	0x07		; Eine Eingabe vom User war falsch
;---------------------------------------

;------ max. Signale bei Level XY ------
S_LEVEL1		equ	0x08		;  8 Signale (Level1) bis Ende
S_LEVEL2		equ	0x0E		; 14 Signale (Level2) bis Ende
S_LEVEL3		equ	0x14		; 20 Signale (Level3) bis Ende
S_LEVEL4		equ	0x1F		; 31 Signale (Level4) bis Ende

S_LEVEL5		equ	0xF0		; Spezialmode1 (4*2 Signale)
S_LEVEL6		equ	0xF1		; Spezialmode2 (8*2 Signale)
;---------------------------------------

;----------- Tne bei LEDs -------------
TON_LED1		equ	0x05		; Ton bei LED1
TON_LED2		equ	0x04		; Ton bei LED2
TON_LED3		equ	0x03		; Ton bei LED3
TON_LED4		equ	0x02		; Ton bei LED4
;---------------------------------------

;------------ Anz-Zeit User ------------
U_SHOW_TIME		equ	0x05		; 5*100ms=0,5s Anzeigedauer der einzelnen Signale bei Usereingaben
U_INPUT_PAUSE		equ	0x05		; 5*1s=5s, max. Zeitdauer zur Eingabe eines Signals
;---------------------------------------

;----------- Anz-Zeit Senso ------------
SHOW_TIME_SG0		equ	0x05		; 0,5s Dauer der einzelnen Signale beim Vorspielen (S_GRAD0)
SHOW_PAUSE_SG0		equ	0x03		; 0,3s Pause zwischen den einzelnen Signalen beim Vorspielen (S_GRAD0)

SHOW_TIME_SG1		equ	0x04		; 0,4s Dauer (S_GRAD1)
SHOW_PAUSE_SG1		equ	0x02		; 0,2s Pause (S_GRAD1)

SHOW_TIME_SG2		equ	0x03		; 0,3s Dauer (S_GRAD2)
SHOW_PAUSE_SG2		equ	0x02		; 0,2s Pause (S_GRAD2)

SHOW_TIME_SG3		equ	0x02		; 0,2s Dauer (S_GRAD3)
SHOW_PAUSE_SG3		equ	0x02		; 0,2s Pause (S_GRAD3)

BLINKZEIT_LED		equ	0x01		; 0,1s LED an, 0,1s LED aus
;---------------------------------------

;----------- Ton bei Fehler ------------
A_TON_FEHLER		equ	0x01		; Anfangs-Ton bei Fehler
E_TON_FEHLER		equ	0x14		; End-Ton bei Fehler
FEHLER_TIME		equ	0x0F		; 1,5s Fehlerton
;---------------------------------------

;----------- Ton bei Erfolg ------------
ERFOLG_TIME		equ	0x05		; 0,5s Lnge Erfolgton
TON1_ERFOLG		equ	0x01		; 1. und 3. Ton bei Erfolg
TON2_ERFOLG		equ	0x05		; 2. Ton bei Erfolg
;---------------------------------------

;########################################################################################
;	Variablen
;########################################################################################

;======= RAM-Bank0 =========================
	cblock	H'0020'	
	rescue_w				; Inhalt des geretteten "w" bei Interrupt (Bank1)
	rescue_status				; Inhalt des geretteten "status" bei Interrupt
	rescue_pclath				; Inhalt des geretteten "pclath" bei Interrupt

	zaehler					; 200us Zhler
	tick					; 1ms Zhler
	zaehler_10ms				; 10ms Zhler
	zaehler_100ms				; 100ms Zhler
	zaehler_1s				; 1s Zhler
	piezo_zaehler				; Toggel-Variable fr Piezo
	random_byte				; Zufallswert fr nchstes Signal
	zaehler_taster1				; Zhler fr Entprellroutine (Taster1)
	zaehler_taster2				; Zhler fr Entprellroutine (Taster2)
	zaehler_taster3				; Zhler fr Entprellroutine (Taster3)
	zaehler_taster4				; Zhler fr Entprellroutine (Taster4)
;	Frei
;	Frei
	endc


	cblock	H'0030'
	piezo_freq_h				; bergabevariable der Anfangs-Tonhhe
	piezo_freq_l				; bergabevariable der End-Tonhhe
	piezo_freq_akt				; aktuelle Frequenz des Piezos
	zeitdauer_led				; wie lange LED aktiv?
	zeitdauer_piezo				; wie lange Piezo aktiv?
	pause_anzeige				; Zeitraum zwischen den einzelnen Signalen
	pause_eingabe				; max. Zeitraum zur Eingabe eines Signals
	show_time				; Anzeigedauer der LEDs beim Vorspielen
	show_pause				; Pause zwischen dem Vorspielen der einzelnen LEDs
	tasten_merker_reg0			; Merker fr Tasten FLANKE (s.u.)
	tasten_merker_reg1			; Merker fr Tasten ENTPRELLT (s.u.)
	merker_reg1				; Merker fr LEDs (s.u.)
	merker_reg2				; Merker fr den Piezo-Summer
	mode					; Merker fr Arbeitszustnde
	level_wahl				; welcher Schwierigkeitsgrad wurde gewhlt?
;	Frei
	endc


	cblock	H'0040'
	ausschalten				; Zeit-Zhler zum Abschalten
	sequence_ende				; Adresse der letzten Position
	max_sequence				; Adresse der max Position von seqence_ende (von Levelwahl abhngig)
	geschwindigkeit				; Merker fr die aktuelle Geschwindigkeit
	temp1					; temporre Variable (fr Vergleiche bentigt)
	temp2					; temporre Variable (Zhler fr ErfolgTon)
	temp3					; Merker fr SayHello und S_Grad_waehlen
	endc

;################ WICHTIG!! ###################
;Bei Benutzung des ICD mit dem PIC 16F876
;sind folgende Speicherbereiche NICHT nutzbar:
;
;File Register:		0x70, 0x1EB-0x1EF
;Programm Memory:	0x1F00-0x1FFF
;##############################################

;===========================================



;########################################################################################
;	Bit-Defines
;########################################################################################

;===== Portpin-Definitionen ================
#define TASTER_1	PORTA, 0		; Taster1
#define TASTER_2	PORTA, 1		; Taster2
#define TASTER_3	PORTA, 2		; Taster3
#define TASTER_4	PORTA, 3		; Taster4

#define	LED_1		PORTB, 0		; LED1 (grn)
#define	LED_2		PORTB, 1		; LED2 (gelb)
#define	LED_3		PORTB, 2		; LED3 (rot)
#define	LED_4		PORTB, 3		; LED4 (blau)

#define PIEZO		PORTC, 2		; Piezo-Lautsprecher

#define	POWER		PORTC, 3		; Power Selbsterhaltung
;===========================================


;===== Merker-Definitionen =================

;----- Elemente von "tasten_merker_reg0" ----
#define	TASTER1_OLD	tasten_merker_reg0,0		; Merker fr Tastenzustnde
#define	TASTER1_PULS	tasten_merker_reg0,1		; TASTER_OLD = Zustand bei letzter Abfrage
#define	TASTER2_OLD	tasten_merker_reg0,2		; TASTER_PULS = Flanke erkannt
#define	TASTER2_PULS	tasten_merker_reg0,3
#define	TASTER3_OLD	tasten_merker_reg0,4
#define	TASTER3_PULS	tasten_merker_reg0,5
#define	TASTER4_OLD	tasten_merker_reg0,6
#define	TASTER4_PULS	tasten_merker_reg0,7
;--------------------------------------------

;----- Elemente von "tasten_merker_reg1" ----
#define	TASTER1_AKTIV	tasten_merker_reg1,0		; Merker fr Tastenzustnde (entprellt)
#define	TASTER2_AKTIV	tasten_merker_reg1,1
#define	TASTER3_AKTIV	tasten_merker_reg1,2
#define	TASTER4_AKTIV	tasten_merker_reg1,3
;--------------------------------------------

;----- Elemente von "merker_reg1" -----
#define	LED1_AKTIV	merker_reg1,0			; Merker fr LED1
#define	LED2_AKTIV	merker_reg1,1			; Merker fr LED2
#define	LED3_AKTIV	merker_reg1,2			; Merker fr LED3
#define	LED4_AKTIV	merker_reg1,3			; Merker fr LED4
#define	ZWEITE		merker_reg1,4			; ist beim Spezialmode schon die zweite LED ausgegeben?
;--------------------------------------

;----- Elemente von "merker_reg2" -----
#define	PIEZO_AKTIV	merker_reg2,0			; Merker fr Piezo-Summer
#define	PIEZO_TOGGLE	merker_reg2,1			; Merker fr toggeln des Piezo (an/aus)
#define PIEZO_TAKTGEBER	merker_reg2,2			; Merker fr toggeln des Piezos (wird alle 100ms getoggelt)
#define	TON_FLANKE_OFF	merker_reg2,3			; Merker fr "Piezo grade aus"
#define PROC_AKTIV	merker_reg2,4			; Ist ErfolgTon oder FehlerTon aktiv?
#define LAST_OUTPUT	merker_reg2,5			; letztes Signal ausgegeben?
#define	WAHL_LEVEL6	merker_reg2,6			; ist bei CHOOSE_LEVEL Level5 gewhlt worden?
#define SPEZIALMODE	merker_reg2,7			; Spezialmode aktiv?
;--------------------------------------

;===========================================


;########################################################################################
;	Byte-Defines
;########################################################################################

#define	DATA_DIRECTION_PORTA	b'00111111'	; Bit 7 und 8 gleich Null, da bei PortA nicht vorhanden
						; A0:	Input:	TASTE1
						; A1:	Input:	TASTE2
						; A2:	Input:	TASTE3
						; A3:	Input:	TASTE4
						; A4:	Input:	%
						; A5:	Input:	%

#define	DATA_DIRECTION_PORTB	b'11110000'	; B0:	Output:	LED1
						; B1:	Output:	LED2
						; B2:	Output:	LED3
						; B3:	Output: LED4
						; B4:	Input:	%
						; B5:	Input:	%
						; B6:	Input:	%	(ICD)
						; B7:	Input:	%	(ICD)

#define	DATA_DIRECTION_PORTC	b'11110011'	; C0:	Input:	%
						; C1:	Input:	%
						; C2:	Output: PIEZO
						; C3:	Output:	Selbsthaltung Power
						; C4:	Input:	%
						; C5:	Input:	%
						; C6:	Input:	%
						; C7:	Input:	%



;########################################################################################
;	Macros
;########################################################################################

	include <macro.asm>

;########################################################################################



;########################################################################################
;########################################################################################
;
;	Sprung zum Programmbeginn
;
;########################################################################################
;########################################################################################

	org	H'0000'
	nop				; Unbedingt bei ICD!
	goto	start


	org	H'0004'
	goto	Interrupt_Service	; ISR-Adresse


;########################################################################################
;########################################################################################
;
;	Modul:			Interrupt-Service-Routine(n)
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Es wird nachgeprft, welcher Interrupt der Auslser war.
;				In diesem Fall gibt es nur einen Auslser:
;				- Timer0 (Tonerzeugung fr den Piezo und Zeitbasis)
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################
;########################################################################################

Interrupt_Service

	;===== Retten wichtiger Register ========
	movwf	rescue_w			; Retten von "w" auf RAM
	swapf	STATUS,w			; Status-Reg gedreht in W (somit kein Flag beeinflut)
	clrf	STATUS				; Bank0
	movwf	rescue_status			; Retten von gedrehtem "status" auf RAM
	movf	PCLATH,w			; "pclath" in W
	movwf	rescue_pclath			; Retten von "pclath" auf RAM
	clrf	PCLATH
	;========================================


	btfss	INTCON,T0IF			; Interrupt durch Timer0?
	goto	isr_end				; nein, gehe zum Ende

	;========== Int. durch Timer0 ===========
	bcf	INTCON,T0IF			; Flag fr neuen Interrupt lschen (Timer0)
	movlw	TIMER0_RELOAD			; Reloadwert setzen
	movwf	TMR0

	incf	zaehler
	movlw	0x05
	xorwf	zaehler,w
	btfss	STATUS,Z
	goto	isr_zaehler_weiter
	incf	tick				; Timebase (1ms)
	clrf	zaehler
isr_zaehler_weiter:

	incf	random_byte			; Zufallswert fr nchstes Signal

	btfss	PIEZO_AKTIV			; PIEZO_AKTIV gesetzt?
	goto	isr_piezo_off			; nein, gehe zum Ende
	movlw	0x00				; Zeitdauer fr Piezo vorbei?
	xorwf	zeitdauer_piezo,w
	btfsc	STATUS,Z
	goto	isr_piezo_off

	movlw	0x00
	xorwf	piezo_zaehler,w
	btfss	STATUS,Z
	decf	piezo_zaehler

	btfss	PIEZO_TOGGLE
	goto	isr_piezo_zeit

isr_piezo_toggle:
	btfss	PIEZO_TAKTGEBER
	goto	isr_piezo_freq_l
	movfw	piezo_freq_h
	movwf	piezo_freq_akt
	goto	isr_piezo_zeit
isr_piezo_freq_l:
	movfw	piezo_freq_l
	movwf	piezo_freq_akt

isr_piezo_zeit:
	movlw	0x00				; nein, Zeitdauer nicht vorbei
	xorwf	piezo_zaehler,w
	btfss	STATUS,Z
	goto	isr_piezo_end
	movfw	piezo_freq_akt
	movwf	piezo_zaehler
	btfss	PIEZO				; Piezo-Ansteuerung
	goto	isr_piezo_on
	bcf	PIEZO
	goto	isr_piezo_end
isr_piezo_on:
	bsf	PIEZO
	goto	isr_piezo_end
isr_piezo_off:
	bcf	PIEZO
	bcf	PIEZO_AKTIV
	
isr_piezo_end:

	;========================================


isr_end:
	;===== Restaurieren der Register ========
	movf	rescue_pclath,w			; Geretteten Inhalt von "pclath" lesen
	movwf	PCLATH				; Restaurieren
	swapf	rescue_status,w			; Geretteten Inhalt von "Status" gedreht zurck in "W"
	movwf	STATUS				; Restaurieren
	swapf	rescue_w,f
	swapf	rescue_w,w
	;========================================

	retfie

;############# Ende: Interrupt-Service-Routine(n) #######################################



;########################################################################################
;
;	Modul:			ConfigController
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Es werden die Datenrichtungsregister und die Startzustnde der
;				Ports definiert. Desweiteren wird Timer0 und Timer2 gestartet,
;				der Interrupt dazu initialisiert und der AD-Wandler deaktiviert.
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

ConfigController

	;===== Datenrichtungen einstellen =======
	clrf	PORTA				; Lschen des Eingangsregisters
	bank_1
	movlw	0x00				; A/D-Wandler von PortA inaktiv
	movwf	ADCON0
	movlw	0x06				; A/D-Wandler von PortA inaktiv
	movwf	ADCON1
	movlw	DATA_DIRECTION_PORTA
	movwf	TRISA				; Datenrichtung festlegen

	bank_0
	clrf	PORTB				; Lschen des Eingangsregisters
	bank_1
	movlw	DATA_DIRECTION_PORTB
	movwf	TRISB				; Datenrichtung festlegen

	bank_0
	clrf	PORTC				; Lschen des Eingangsregisters
	bank_1
	movlw	DATA_DIRECTION_PORTC
	movwf	TRISC				; Datenrichtung festlegen

	bank_0
	;========================================


	;======== Timer0 initialisieren =========
	bank_1
	movfw	OPTION_REG
	andlw	0xC0				; setzen der Maske (b: 11000000)
	movwf	OPTION_REG			; Prescaler 1:2
	bank_0

	bsf	INTCON,T0IE			; Timer0 Interrupt Enable
	bsf	INTCON,PEIE			; Peripheral Interrupts Enable
	bsf	INTCON,GIE			; Global Interrupt Enable

	clrf	TMR0				; Timer0-Register lschen
	;========================================

	return

;############# Ende: ConfigController ###################################################




;########################################################################################
;
;	Modul:			ClearRAM
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Der Arbeitsspeicher wird komplett gelscht (Inhalt = 0x00)
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:		bank 0
;
;	Speicherseite:		page 0
;
;	Hinweise:		
;
;########################################################################################

ClearRAM

	;===== Lschen des RAMs in Bank0 ========
	movlw	H'0020'				; Lschen der RAM-speicherzellen
	movwf	FSR				; Starte bei Adresse 20, bank 0 (Zeigerregister Indirekt laden)


clr_b0	clrf	INDF				; Lsche Register an Adresse "fsr"
	incf	FSR				; Nchste Adresse
	btfss	FSR, 7				; Bei Adresse 0x80 angekommen?
	goto	clr_b0				; Wenn nicht, dann nochmal

	;===== Lschen des RAMs in Bank1 ========
	movlw	H'00A0'				; Lschen der RAM-Speicherzellen
	movwf	FSR				; Weiter bei Adresse 0xA0 in bank 1

clr_b1	clrf	INDF				; Lsche Register an Adresse "fsr"
	incfsz	FSR				; berlauf erkannt (>0xFF)?
	goto	clr_b1				; wenn nicht, dann nochmal

	;===== Lschen des RAMs in Bank2 ========

	;===== Lschen des RAMs in Bank3 ========

	return

;############# Ende: ClearRAM ###########################################################



;########################################################################################
;
;	Modul:			Hochfahren
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Setzt Selbsthaltung nur bei Puls von Taster4
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:		bank 0
;
;	Speicherseite:		page 0
;
;	Hinweise:		
;
;########################################################################################

Hochfahren

	btfss	TASTER4_PULS
	goto	hf_ende

	;======== Power Selbsterhaltung =========
	bsf	POWER				; Selbsterhaltung (PIN auf Low)
	;========================================

	movlw	ANIMATION
	movwf	mode				; Mode setzen
hf_ende:

	return

;############# Ende: Hochfahren #########################################################



;########################################################################################
;
;	Modul:			SayHello
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Lsst die LEDs einmal von links nach rechts und zurck
;				aufblinken
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

SayHello

	movlw	0x00
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_led2
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED1_AKTIV
	goto	sh_ende

sh_led2:
	movlw	0x01
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_led3
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED2_AKTIV
	goto	sh_ende

sh_led3:
	movlw	0x02
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_led4
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED3_AKTIV
	goto	sh_ende

sh_led4:
	movlw	0x03
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_led5
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED4_AKTIV
	goto	sh_ende

sh_led5:
	movlw	0x04
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_led6
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED3_AKTIV
	goto	sh_ende

sh_led6:
	movlw	0x05
	xorwf	temp3,w
	btfss	STATUS,Z
	goto	sh_pause_abwarten
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende
	movlw	0x01
	movwf	zeitdauer_led
	incf	temp3
	clrf	merker_reg1
	bsf	LED2_AKTIV
	goto	sh_ende

sh_pause_abwarten:
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	sh_ende

sh_change_mode:
	clrf	merker_reg1
	movlw	CHOOSE_LEVEL
	movwf	mode				; Mode setzen
	movlw	0x00
	movwf	temp3				; Variable zurcksetzen	

;--- Einstellung fr CHOOSE_LEVEL ---
	bsf	LED1_AKTIV
	movlw	S_LEVEL1
	movwf	level_wahl			; Schwierigkeitsgrad1 voreingestellt
	movlw	0x00
	movwf	temp3				; temporre Variable zurcksetzen
;------------------------------------

sh_ende:
	movlw	0x00

	return

;############# Ende: SayHello ###########################################################



;########################################################################################
;
;	Modul:			S_Grad_waehlen
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		User whlt einen Schwierigkeitsgrad aus (Level 1-4)
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:		bank 0
;
;	Speicherseite:		page 0
;
;	Hinweise:		
;
;########################################################################################

S_Grad_waehlen

	btfss	TASTER1_PULS			; Taster1 gedrckt?
	goto	sgw_taster2			; nein

	movlw	S_LEVEL1
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level1 gesetzt?
	goto	sgw_pruef2			; nein
	movlw	S_LEVEL2			; ja, setze nun Level2
	movwf	level_wahl
	clrf	merker_reg1
	bsf	LED2_AKTIV			; setze LED2 aktiv
	goto	sgw_ende

sgw_pruef2:
	movlw	S_LEVEL2
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level2 gesetzt?
	goto	sgw_pruef3			; nein
	movlw	S_LEVEL3			; ja, setze nun Level3
	movwf	level_wahl
	clrf	merker_reg1
	bsf	LED3_AKTIV			; setze LED3 aktiv
	goto	sgw_ende

sgw_pruef3:
	movlw	S_LEVEL3
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level3 gesetzt?
	goto	sgw_pruef4			; nein
	movlw	S_LEVEL4			; ja, setze nun Level4
	movwf	level_wahl
	clrf	merker_reg1
	bsf	LED4_AKTIV			; setze LED4 aktiv
	goto	sgw_ende

sgw_pruef4:
	movlw	S_LEVEL4
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level4 gesetzt?
	goto	sgw_pruef5			; nein
	movlw	S_LEVEL5			; ja, setze nun Level5
	movwf	level_wahl
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	LED2_AKTIV			; setze LED1+LED2 aktiv
	goto	sgw_ende

sgw_pruef5:
	movlw	S_LEVEL5
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level5 gesetzt?
	goto	sgw_pruef6			; nein
	movlw	S_LEVEL6			; ja, setze nun Level6
	movwf	level_wahl

	bsf	WAHL_LEVEL6
	bsf	LED1_AKTIV
	bsf	LED2_AKTIV
	bsf	LED3_AKTIV
	bsf	LED4_AKTIV			; setze alle LEDs aktiv
	goto	sgw_ende

sgw_pruef6:
	movlw	S_LEVEL6
	xorwf	level_wahl,w
	btfss	STATUS,Z			; Level6 gesetzt?
	goto	sgw_ende			; nein
	movlw	S_LEVEL1			; ja, setze nun wieder Level1
	movwf	level_wahl
	clrf	merker_reg1
	bsf	LED1_AKTIV			; setze LED1 aktiv
	goto	sgw_ende


sgw_taster2:
	btfss	TASTER4_PULS			; Taster4 gedrckt?
	goto	sgw_ende			; nein
	
	movlw	0x08				; Taster4 wurde gedrckt (S_Grad wurde gewhlt)
	movwf	pause_anzeige			; 0,7s warten

	movlw	S_LEVEL5
	xorwf	level_wahl,w			; Spezialmode1 (Level5) gesetzt?
	btfss	STATUS,Z
	goto	sg_pruef2			; nein, prfen auf LEVEL6
	movlw	0xA9				; ja, Spezialmode gesetzt
	movwf	max_sequence			; nach 4*2=8 Signalen gewonnen
	goto	sg_spm_setzen
sg_pruef2:
	movlw	S_LEVEL6
	xorwf	level_wahl,w
	btfss	STATUS,Z
	goto	sg_setzen			; nein, normal weiter
	movlw	0xB1
	movwf	max_sequence			; nach 8*2=16 Signalen gewonnen
sg_spm_setzen:
	bsf	SPEZIALMODE			; Merker setzen!!
	movlw	0xA0
	movwf	FSR				; FSR setzen fr Sequenz-Speicherung
	movfw	random_byte
	andlw	0x03				; setzen der Maske (b: 00000011)
	movwf	INDF				; ersten Zufallswert schreiben

sg_zufall_nochmal:				; VERHINDERN, DASS ZWEIMAL GLEICHES SIGNAL!!
	movlw	0xA0
	movwf	FSR
	movfw	INDF
	movwf	temp3
	incf	FSR

	swapf	random_byte			; zweites Signal setzen
	movfw	random_byte
	andlw	0x03
	xorwf	temp3,f
	btfsc	STATUS,Z			; W gleich temp3?
	goto	sg_zufall_nochmal		; ja, nochmal (neuer Zufallswert)
	movwf	INDF				; zweiter Zufallswert schreiben

	movlw	0xA1
	movwf	sequence_ende			; Adresse des letzten Signals

	movlw	0xA0
	movwf	FSR

	goto	sg_einst_weiter		
sg_setzen:
	movfw	level_wahl
	addlw	0xA0
	movwf	max_sequence			; setzen der max. Folge (nach Levelwahl)
	movlw	0xA0
	movwf	FSR				; FSR setzen fr Sequenz-Speicherung
	movwf	sequence_ende			; Adresse des letzten Signals

	movfw	random_byte
	andlw	0x03				; setzen der Maske (b: 00000011)
	movwf	INDF				; ersten Zufallswert schreiben	

sg_einst_weiter:
	movlw	SHOW_TIME_SG0
	movwf	show_time			; Anzeigedauer der einzelnen Signale bei Vorspielen setzen
	movlw	SHOW_PAUSE_SG0
	movwf	show_pause			; Pause zwischen den einzelnen Farben bei Vorspielen setzen
	movlw	0x00
	movwf	geschwindigkeit			; Abspielgeschwindigkeit setzen

	clrf	PORTB
	clrf	merker_reg1			; alle LEDs auf aus!

	movlw	WAIT_OUTPUT
	movwf	mode				; Mode setzen

sgw_ende:

	return

;############# Ende: S_Grad_waehlen #####################################################


;########################################################################################
;
;	Modul:			TimeBase
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Erzeugt aus dem "Tick" (wird jede ms inkrementiert) einen 10ms, einen 100ms und einen 1s Counter
;				Desweiteren wird die Variable "zeitdauer_piezo" und "zeitdauer_led" dekrementiert, wenn !=0
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

TimeBase

	movlw	0x09
	xorwf	tick,w
	btfss	STATUS,Z			; 1ms-Zhler > 9?
	goto	tb_weiter1

	;============ 10ms vergangen ============
	clrf	tick
	incf	zaehler_10ms

	btfsc	PIEZO_TOGGLE
	goto	tb_weiter1
	btfss	zaehler_10ms,1
	goto	tb_weiter1
	movfw	piezo_freq_l
	xorwf	piezo_freq_akt,w
	btfsc	STATUS,Z
	goto	tb_weiter1
	incf	piezo_freq_akt
	;========================================

tb_weiter1:
	movlw	0x0A
	xorwf	zaehler_10ms,w
	btfss	STATUS,Z			; 10ms-Zhler > 9?
	goto	tb_weiter2

	;=========== 100ms vergangen ============
	clrf	zaehler_10ms
	incf	zaehler_100ms

;#####
	bcf	TON_FLANKE_OFF
	movlw	0x01
	xorwf	zeitdauer_piezo,w
	btfsc	STATUS,Z
	bsf	TON_FLANKE_OFF

	movlw	0x00
	xorwf	zeitdauer_piezo,w
	btfss	STATUS,Z
	decf	zeitdauer_piezo
;#####

	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	decf	zeitdauer_led


	movlw	0x00
	xorwf	pause_anzeige,w
	btfss	STATUS,Z
	decf	pause_anzeige

	btfss	PIEZO_TAKTGEBER
	goto	tb_ptakt_an
	bcf	PIEZO_TAKTGEBER
	goto	tb_weiter2
tb_ptakt_an:
	bsf	PIEZO_TAKTGEBER
	;========================================

tb_weiter2:
	movlw	0x0A
	xorwf	zaehler_100ms,w
	btfss	STATUS,Z			; 100ms-Zhler > 9?
	goto	tb_weiter3

	;============= 1s vergangen =============
	clrf	zaehler_100ms
	incf	zaehler_1s

	movlw	0x00
	xorwf	pause_eingabe,w
	btfss	STATUS,Z
	decf	pause_eingabe

	movlw	0x00
	xorwf	ausschalten,w
	btfsc	STATUS,Z
	goto	schalten
	decf	ausschalten
	goto	tb_weiter3
schalten:
	bcf	POWER
	;========================================

tb_weiter3:

	return

;############# Ende: TimeBase ###########################################################



;########################################################################################
;
;	Modul:			Taster_lesen
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Einlesen der Taster (mit Entprellroutine)
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

Taster_lesen

;------------------ Taster1 ------------------
	bcf	TASTER1_PULS			; kein Puls
	btfss	TASTER_1			; Taster1 gedrckt?
	goto	taster1_off			; nein, Taster1 nicht gedrckt
	btfsc	TASTER1_OLD			; war Taster1 gedrckt?
	goto	taster1_gedrueckt		; ja, Taster1 war gedrckt
	clrf	zaehler_taster1			; nein, Taster1 war nicht gedrckt
	goto	t_zaehler1_pruef
taster1_gedrueckt:
	incf	zaehler_taster1
t_zaehler1_pruef:
	bsf	TASTER1_OLD			; Speicherung "alter Zustand"
	movlw	0x0F				; Zhler=15?
	xorwf	zaehler_taster1,w
	btfss	STATUS,Z
	goto	t_zaehler1_pruef2		; nein
	bsf	TASTER1_PULS			; setzen der Flanke fr einen Zyklus
	movlw 	AUSSCHALTZEIT
	movwf	ausschalten			; wieder Ausschaltzeit neu setzen
t_zaehler1_pruef2:
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster1,w
	btfss	STATUS,Z
	goto	taster1_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster1			; Zhler auf 15 begrenzen
	bsf	TASTER1_AKTIV
	goto	taster1_ende

taster1_off:
	btfss	TASTER1_OLD			; war Taster1 bereits NICHT gedrckt?
	goto	taster1_nicht_gedrueckt		; ja, Taster1 war NICHT gedrckt
	clrf	zaehler_taster1
	goto	t_zaehler1_pruef3
taster1_nicht_gedrueckt:
	incf	zaehler_taster1
t_zaehler1_pruef3:
	bcf	TASTER1_OLD			; Speicherung "alter Zustand"
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster1,w
	btfss	STATUS,Z
	goto	taster1_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster1			; Zhler auf 15 begrenzen
	bcf	TASTER1_AKTIV
taster1_ende:
;---------------------------------------------


;------------------ Taster2 ------------------
	bcf	TASTER2_PULS			; kein Puls
	btfsc	TASTER_2			; Taster2 gedrckt?
	goto	taster2_off			; nein, Taster2 nicht gedrckt
	btfsc	TASTER2_OLD			; war Taster2 gedrckt?
	goto	taster2_gedrueckt		; ja, Taster2 war gedrckt
	clrf	zaehler_taster2			; nein, Taster2 war nicht gedrckt
	goto	t_zaehler2_pruef
taster2_gedrueckt:
	incf	zaehler_taster2
t_zaehler2_pruef:
	bsf	TASTER2_OLD			; Speicherung "alter Zustand"
	movlw	0x0F				; Zhler=15?
	xorwf	zaehler_taster2,w
	btfss	STATUS,Z
	goto	t_zaehler2_pruef2		; nein
	bsf	TASTER2_PULS			; setzen der Flanke fr einen Zyklus
	movlw 	AUSSCHALTZEIT
	movwf	ausschalten			; wieder Ausschaltzeit neu setzen
t_zaehler2_pruef2:
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster2,w
	btfss	STATUS,Z
	goto	taster2_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster2			; Zhler auf 15 begrenzen
	bsf	TASTER2_AKTIV
	goto	taster2_ende

taster2_off:
	btfss	TASTER2_OLD			; war Taster2 bereits NICHT gedrckt?
	goto	taster2_nicht_gedrueckt		; ja, Taster2 war NICHT gedrckt
	clrf	zaehler_taster2
	goto	t_zaehler2_pruef3
taster2_nicht_gedrueckt:
	incf	zaehler_taster2
t_zaehler2_pruef3:
	bcf	TASTER2_OLD			; Speicherung "alter Zustand"
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster2,w
	btfss	STATUS,Z
	goto	taster2_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster2			; Zhler auf 15 begrenzen
	bcf	TASTER2_AKTIV
taster2_ende:
;---------------------------------------------


;------------------ Taster3 ------------------
	bcf	TASTER3_PULS			; kein Puls
	btfsc	TASTER_3			; Taster3 gedrckt?
	goto	taster3_off			; nein, Taster3 nicht gedrckt
	btfsc	TASTER3_OLD			; war Taster3 gedrckt?
	goto	taster3_gedrueckt		; ja, Taster3 war gedrckt
	clrf	zaehler_taster3			; nein, Taster3 war nicht gedrckt
	goto	t_zaehler3_pruef
taster3_gedrueckt:
	incf	zaehler_taster3
t_zaehler3_pruef:
	bsf	TASTER3_OLD			; Speicherung "alter Zustand"
	movlw	0x0F				; Zhler=15?
	xorwf	zaehler_taster3,w
	btfss	STATUS,Z
	goto	t_zaehler3_pruef2		; nein
	bsf	TASTER3_PULS			; setzen der Flanke fr einen Zyklus
	movlw 	AUSSCHALTZEIT
	movwf	ausschalten			; wieder Ausschaltzeit neu setzen
t_zaehler3_pruef2:
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster3,w
	btfss	STATUS,Z
	goto	taster3_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster3			; Zhler auf 15 begrenzen
	bsf	TASTER3_AKTIV
	goto	taster3_ende

taster3_off:
	btfss	TASTER3_OLD			; war Taster3 bereits NICHT gedrckt?
	goto	taster3_nicht_gedrueckt		; ja, Taster3 war NICHT gedrckt
	clrf	zaehler_taster3
	goto	t_zaehler3_pruef3
taster3_nicht_gedrueckt:
	incf	zaehler_taster3
t_zaehler3_pruef3:
	bcf	TASTER3_OLD			; Speicherung "alter Zustand"
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster3,w
	btfss	STATUS,Z
	goto	taster3_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster3			; Zhler auf 15 begrenzen
	bcf	TASTER3_AKTIV
taster3_ende:
;---------------------------------------------


;------------------ Taster4 ------------------
	bcf	TASTER4_PULS			; kein Puls
	btfsc	TASTER_4			; Taster4 gedrckt?
	goto	taster4_off			; nein, Taster4 nicht gedrckt
	btfsc	TASTER4_OLD			; war Taster4 gedrckt?
	goto	taster4_gedrueckt		; ja, Taster4 war gedrckt
	clrf	zaehler_taster4			; nein, Taster4 war nicht gedrckt
	goto	t_zaehler4_pruef
taster4_gedrueckt:
	incf	zaehler_taster4
t_zaehler4_pruef:
	bsf	TASTER4_OLD			; Speicherung "alter Zustand"
	movlw	0x0F				; Zhler=15?
	xorwf	zaehler_taster4,w
	btfss	STATUS,Z
	goto	t_zaehler4_pruef2		; nein
	bsf	TASTER4_PULS			; setzen der Flanke fr einen Zyklus
	movlw 	AUSSCHALTZEIT
	movwf	ausschalten			; wieder Ausschaltzeit neu setzen
t_zaehler4_pruef2:
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster4,w
	btfss	STATUS,Z
	goto	taster4_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster4			; Zhler auf 15 begrenzen
	bsf	TASTER4_AKTIV
	goto	taster4_ende

taster4_off:
	btfss	TASTER4_OLD			; war Taster4 bereits NICHT gedrckt?
	goto	taster4_nicht_gedrueckt		; ja, Taster4 war NICHT gedrckt
	clrf	zaehler_taster4
	goto	t_zaehler4_pruef3
taster4_nicht_gedrueckt:
	incf	zaehler_taster4
t_zaehler4_pruef3:
	bcf	TASTER4_OLD			; Speicherung "alter Zustand"
	movlw	0x10				; Zhler=16?
	xorwf	zaehler_taster4,w
	btfss	STATUS,Z
	goto	taster4_ende			; nein, gehe zum Ende
	movlw	0x0F
	movwf	zaehler_taster4			; Zhler auf 15 begrenzen
	bcf	TASTER4_AKTIV
taster4_ende:
;---------------------------------------------

	return

;############# Ende: Taster_lesen ###################################################



;########################################################################################
;
;	Modul:			SequenceOutput
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Senso gibt die (bisherige) Signalfolge aus
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

SequenceOutput

;-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
	btfss	SPEZIALMODE
	goto	so_normal

	btfsc	TON_FLANKE_OFF
	goto	so_status_warten		; Ton grade abgelaufen --> Pause machen
	movlw	0x00				; Ton nicht grade abgelaufen
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	so_ende				; Ton luft noch
	movlw	0x01				; FSR=sequence+1?
	subwf	FSR,0
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	so_sp_ausgabe			; Signal ausgeben
	movlw	0xA0				; alles ausgegeben --> ENDE
	movwf	FSR				; FSR auf Anfang zurcksetzen
	bsf	LAST_OUTPUT
	bcf	PIEZO_TOGGLE
	movlw	WAIT_OUTPUT
	movwf	mode
	goto	so_ende

so_sp_ausgabe:
	bsf	PIEZO_TOGGLE
	movfw	INDF				; lese aktuelle Sequenz
	movwf	temp1				; entscheide welche LED
	movlw	0x00
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_sp_test_led2
	btfsc	ZWEITE				; wurde eine LED schon ausgegeben?
	goto	so_sp_led1			; ja
	clrf	merker_reg1
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	bsf	ZWEITE
	bsf	LED1_AKTIV
	incf	FSR
	goto	so_ende
so_sp_led1:
	movlw	TON_LED1
	movwf	piezo_freq_l
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	bsf	LED1_AKTIV
	bsf	PIEZO_AKTIV
	bcf	ZWEITE
	incf	FSR
	goto	so_sp_test_led_ende

so_sp_test_led2:
	movfw	INDF
	movwf	temp1
	movlw	0x01
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_sp_test_led3
	btfsc	ZWEITE				; wurde eine LED schon ausgegeben?
	goto	so_sp_led2			; ja
	clrf	merker_reg1
	movlw	TON_LED2
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	bsf	ZWEITE
	bsf	LED2_AKTIV
	incf	FSR
	goto	so_ende
so_sp_led2:
	movlw	TON_LED2
	movwf	piezo_freq_l
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	bsf	LED2_AKTIV
	bsf	PIEZO_AKTIV
	bcf	ZWEITE
	incf	FSR
	goto	so_sp_test_led_ende

so_sp_test_led3:
	movfw	INDF
	movwf	temp1
	movlw	0x02
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_sp_test_led4
	btfsc	ZWEITE				; wurde eine LED schon ausgegeben?
	goto	so_sp_led3			; ja
	clrf	merker_reg1
	movlw	TON_LED3
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	bsf	ZWEITE
	bsf	LED3_AKTIV
	incf	FSR
	goto	so_ende
so_sp_led3:
	movlw	TON_LED3
	movwf	piezo_freq_l
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	bsf	LED3_AKTIV
	bsf	PIEZO_AKTIV
	bcf	ZWEITE
	incf	FSR
	goto	so_sp_test_led_ende

so_sp_test_led4:
	movfw	INDF
	movwf	temp1
	movlw	0x03
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_sp_test_led_ende
	btfsc	ZWEITE				; wurde eine LED schon ausgegeben?
	goto	so_sp_led4			; ja
	clrf	merker_reg1
	movlw	TON_LED4
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	bsf	ZWEITE
	bsf	LED4_AKTIV
	incf	FSR
	goto	so_ende
so_sp_led4:
	movlw	TON_LED4
	movwf	piezo_freq_l
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV
	incf	FSR
	bcf	ZWEITE

so_sp_test_led_ende:



	goto	so_ende

so_normal:
;-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#


	btfsc	TON_FLANKE_OFF
	goto	so_status_warten		; Ton grade abgelaufen --> Pause machen
	movlw	0x00				; Ton nicht grade abgelaufen
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z
	goto	so_ende				; Ton luft noch
	movlw	0x01				; FSR=sequence+1?
	subwf	FSR,0
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	so_ausgabe			; Signal ausgeben
	movlw	0xA0				; alles ausgegeben --> ENDE
	movwf	FSR				; FSR auf Anfang zurcksetzen
	bsf	LAST_OUTPUT
	movlw	WAIT_OUTPUT
	movwf	mode
	goto	so_ende

so_ausgabe:
	movlw	0xA5				; 6. Signal einer Sequenz erreicht?
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	seq_pruef2
	movlw	0x01
	movwf	geschwindigkeit			; Geschwindigkeit1 setzen
	goto	seq_pruef_end

seq_pruef2:
	movlw	0xAA				; 10. Signal einer Sequenz erreicht?
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	seq_pruef3
	movlw	0x02
	movwf	geschwindigkeit			; Geschwindigkeit2 setzen
	goto	seq_pruef_end

seq_pruef3:
	movlw	0xAE				; 14. Signal einer Sequenz erreicht?
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	seq_pruef_end
	movlw	0x03
	movwf	geschwindigkeit			; Geschwindigkeit3 setzen

seq_pruef_end:
	movlw	0x01
	xorwf	geschwindigkeit,w
	btfss	STATUS,Z
	goto	seq_pruef4
	movlw	SHOW_TIME_SG1
	movwf	show_time
	movlw	SHOW_PAUSE_SG1
	movwf	show_pause			; S_GRAD1 setzen
	goto	seq_pruef_end2
seq_pruef4:
	movlw	0x02
	xorwf	geschwindigkeit,w
	btfss	STATUS,Z
	goto	seq_pruef5
	movlw	SHOW_TIME_SG2
	movwf	show_time
	movlw	SHOW_PAUSE_SG2
	movwf	show_pause			; S_GRAD2 setzen
	goto	seq_pruef_end2
seq_pruef5:
	movlw	0x03
	xorwf	geschwindigkeit,w
	btfss	STATUS,Z
	goto	seq_pruef_end2
	movlw	SHOW_TIME_SG3
	movwf	show_time
	movlw	SHOW_PAUSE_SG3
	movwf	show_pause			; S_GRAD3 setzen

seq_pruef_end2:
	movfw	INDF				; lese aktuelle Sequenz
	movwf	temp1				; entscheide welche LED
	movlw	0x00
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_test_led2
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	PIEZO_AKTIV
	goto	so_test_led_ende


so_test_led2:
	movfw	INDF
	movwf	temp1
	movlw	0x01
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_test_led3
	movlw	TON_LED2
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED2_AKTIV
	bsf	PIEZO_AKTIV
	goto	so_test_led_ende


so_test_led3:
	movfw	INDF
	movwf	temp1
	movlw	0x02
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_test_led4
	movlw	TON_LED3
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED3_AKTIV
	bsf	PIEZO_AKTIV
	goto	so_test_led_ende


so_test_led4:
	movfw	INDF
	movwf	temp1
	movlw	0x03
	xorwf	temp1,w
	btfss	STATUS,Z
	goto	so_test_led_ende
	movlw	TON_LED4
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movfw	show_time
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV


so_test_led_ende:
	incf	FSR				; erhhe Zeigerstand
	goto	so_ende

so_status_warten:
	movlw	WAIT_OUTPUT
	movwf	mode				; Mode setzen
	movfw	show_pause
	movwf	pause_anzeige
	movlw	0x00
	movwf	temp1				; Variable zurcksetzen
	goto	so_ende

so_ende:
	return

;############# Ende: SequenceOutput #####################################################


;########################################################################################
;
;	Modul:			UserInput
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Der User muss die (bisherige) Signalfolge wiederholen
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

UserInput

;-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
	btfss	SPEZIALMODE
	goto	ui_normal

	movlw	0x01				; FSR=sequence+1?
	subwf	FSR,0
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	ui_sp_zeitabfrage		; nein, also zur Zeitabfrage
	incf 	sequence_ende			; ja, User hat die bisherige Folge fehlerfrei geschafft!
	incf	sequence_ende			; nochmal erhhen (Spezialmode)
	movfw	random_byte
	andlw	0x03				; setzen der Maske (b: 00000011)
	movwf	INDF
	movfw	FSR
	movwf	temp1
ui_sp_zufall_nochmal:				; VERHINDERN, DASS ZWEIMAL GLEICHES SIGNAL!!
	movfw	temp1
	movwf	FSR
	movfw	INDF
	movwf	temp3
	incf	FSR

	swapf	random_byte
	movfw	random_byte			; zweites Signal setzen
	andlw	0x03
	xorwf	temp3,f
	btfsc	STATUS,Z			; W gleich temp3?
	goto	ui_sp_zufall_nochmal		; ja, nochmal (neuer Zufallswert)
	movwf	INDF				; zweiter Zufallswert schreiben

	movfw	max_sequence
	xorwf	FSR,w
	btfss	STATUS,Z			; max. Signalfolge erreicht?
	goto	ui_change_mode			; nein

	bcf	PIEZO_TOGGLE			; Piezo-Frequenzen nicht mehr toggeln
	bcf	SPEZIALMODE
	movlw	0x00
	movwf	temp1
	movwf	temp2
	movwf	temp3				; Variablen zurcksetzen!
	movlw	U_SUCCESS
	movwf	mode				; Mode setzen
	goto	userinput_ende



ui_sp_zeitabfrage:
	bsf	PIEZO_TOGGLE			; Piezo-Frequenzen toggeln
	movlw	0x00
	xorwf	pause_eingabe,w
	btfsc	STATUS,Z
	goto	ui_fehler			; Wartezeit vorbei --> Fehler!

;#### Beginn Tastenabfrage
	btfss	TASTER1_PULS			; Taster1 gerade gedrckt?
	goto	ui_sp_taster2			; nein
	btfsc	TASTER2_AKTIV			; Taster2 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen2
	btfsc	TASTER3_AKTIV			; Taster3 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen5
	btfsc	TASTER4_AKTIV			; Taster4 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen3
ui_sp_taster2:
	btfss	TASTER2_PULS			; Taster2 gerade gedrckt?
	goto	ui_sp_taster3			; nein
	btfsc	TASTER1_AKTIV			; Taster1 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen2
	btfsc	TASTER3_AKTIV			; Taster3 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen4
	btfsc	TASTER4_AKTIV			; Taster4 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen6
ui_sp_taster3:
	btfss	TASTER3_PULS			; Taster3 gerade gedrckt?
	goto	ui_sp_taster4			; nein
	btfsc	TASTER1_AKTIV			; Taster1 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen5
	btfsc	TASTER2_AKTIV			; Taster2 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen4
	btfsc	TASTER4_AKTIV			; Taster4 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen1
ui_sp_taster4:
	btfss	TASTER4_PULS			; Taster4 gerade gedrckt?
	goto	ui_sp_tp_ende			; nein
	btfsc	TASTER1_AKTIV			; Taster1 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen3
	btfsc	TASTER2_AKTIV			; Taster2 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen6
	btfsc	TASTER3_AKTIV			; Taster3 gedrckt (und immer noch gehalten!)?
	goto	tasten_pruefen1
	goto	ui_sp_tp_ende

tasten_pruefen1:				; Taster 3+4 wurden gedrckt
	movlw	0x02
	xorwf	INDF,w
	btfss	STATUS,Z			; Taster3 richtig?
	goto	tp1_wechsel			; nein, andere Stelle prfen!
	movlw	0x03
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z			; Taster4 richtig?
	goto	ui_fehler			; nein --> Fehler!
	incf	FSR
	goto	ui_tp1_richtig			; Kombination richtig, gehe zur Ausgabe
tp1_wechsel:	
	movlw	0x03				; Taster4 richtig?
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler			; nein --> Fehler
	movlw	0x02
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z			; Taster3 richtig?
	goto	ui_fehler			; nein --> Fehler
	incf	FSR				; Kombination richtig, gehe zur Ausgabe
ui_tp1_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED3
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED4
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED3_AKTIV
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_sp_tp_ende

tasten_pruefen2:				; Taster 1+2 wurden gedrckt
	movlw	0x00
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	tp2_wechsel
	movlw	0x01
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
	goto	ui_tp2_richtig
tp2_wechsel:	
	movlw	0x01
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	movlw	0x00
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
ui_tp2_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED2
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	LED2_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_sp_tp_ende

tasten_pruefen3:				; Taster 1+4 wurden gedrckt
	movlw	0x00
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	tp3_wechsel
	movlw	0x03
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
	goto	ui_tp3_richtig
tp3_wechsel:	
	movlw	0x03
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	movlw	0x00
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
ui_tp3_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED4
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_sp_tp_ende

tasten_pruefen4:				; Taster 2+3 wurden gedrckt
	movlw	0x01
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	tp4_wechsel
	movlw	0x02
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
	goto	ui_tp4_richtig
tp4_wechsel:	
	movlw	0x02
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	movlw	0x01
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
ui_tp4_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED2
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED3
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED2_AKTIV
	bsf	LED3_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_sp_tp_ende

tasten_pruefen5:				; Taster 1+3 wurden gedrckt
	movlw	0x00
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	tp5_wechsel
	movlw	0x02
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
	goto	ui_tp5_richtig
tp5_wechsel:	
	movlw	0x02
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	movlw	0x00
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
ui_tp5_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED3
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	LED3_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_sp_tp_ende

tasten_pruefen6:				; Taster 2+4 wurden gedrckt
	movlw	0x01
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	tp6_wechsel
	movlw	0x03
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
	goto	ui_tp6_richtig
tp6_wechsel:	
	movlw	0x03
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	movlw	0x01
	incf	FSR
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler
	incf	FSR
ui_tp6_richtig:
	movlw	U_INPUT_PAUSE			; richtige Tasten, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED2
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movlw	TON_LED4
	movwf	piezo_freq_l
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED2_AKTIV
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV

ui_sp_tp_ende:

	goto	userinput_ende
ui_normal:

;-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#



	movlw	0x01				; FSR=sequence+1?
	subwf	FSR,0
	xorwf	sequence_ende,w
	btfss	STATUS,Z
	goto	ui_zeitabfrage			; nein, also zur Zeitabfrage
	incf 	sequence_ende			; ja, User hat die bisherige Folge fehlerfrei geschafft!
	movfw	random_byte
	andlw	0x03				; setzen der Maske (b: 00000011)
	movwf	INDF

	movfw	max_sequence
	xorwf	FSR,w
	btfss	STATUS,Z			; max. Signalfolge erreicht?
	goto	ui_change_mode			; nein
	movlw	0x00
	movwf	temp1
	movwf	temp2
	movwf	temp3				; Variablen zurcksetzen!
	movlw	U_SUCCESS
	movwf	mode				; Mode setzen
	goto	userinput_ende

ui_change_mode:
	movlw	0xA0				; Level noch nicht geschafft
	movwf	FSR
	movlw	0x0A
	movwf	pause_anzeige			; 1s Pause machen bevor SEQUENCE_OUTPUT
	movlw	WAIT_OUTPUT
	movwf	mode				; Mode setzen
	goto	userinput_ende

ui_zeitabfrage:
	movlw	0x00
	xorwf	pause_eingabe,w
	btfsc	STATUS,Z
	goto	ui_fehler

	btfss	TASTER1_PULS			; wurde Taster1 gedrckt?
	goto	ui_taste2
	movlw	0x00				; berprfung: richtige Taste?
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler			; falsche Taste gedrckt

	movlw	U_INPUT_PAUSE			; richtige Taste, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED1
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED1_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_tastepruefen_ende

ui_taste2:
	btfss	TASTER2_PULS			; wurde Taster2 gedrckt?
	goto	ui_taste3

	movlw	0x01				; berprfung: richtige Taste?
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler			; falsche Taste gedrckt

	movlw	U_INPUT_PAUSE			; richtige Taste, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED2
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED2_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_tastepruefen_ende

ui_taste3:
	btfss	TASTER3_PULS			; wurde Taster3 gedrckt?
	goto	ui_taste4

	movlw	0x02				; berprfung: richtige Taste?
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler			; falsche Taste gedrckt

	movlw	U_INPUT_PAUSE			; richtige Taste, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED3
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED3_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_tastepruefen_ende

ui_taste4:
	btfss	TASTER4_PULS			; wurde Taster4 gedrckt?
	goto	userinput_ende

	movlw	0x03				; berprfung: richtige Taste?
	xorwf	INDF,w
	btfss	STATUS,Z
	goto	ui_fehler			; falsche Taste gedrckt

	movlw	U_INPUT_PAUSE			; richtige Taste, Ausgabe vorbereiten
	movwf	pause_eingabe
	movlw	TON_LED4
	movwf	piezo_freq_h
	movwf	piezo_freq_l
	movwf	piezo_freq_akt
	movlw	U_SHOW_TIME
	movwf	zeitdauer_piezo
	movwf	zeitdauer_led
	clrf	merker_reg1
	bsf	LED4_AKTIV
	bsf	PIEZO_AKTIV
	goto	ui_tastepruefen_ende

ui_tastepruefen_ende:
	incf	FSR
	goto	userinput_ende

ui_fehler:
	bcf	PIEZO_TOGGLE			; Piezo-Frequenzen nicht mehr toggeln
	bcf	SPEZIALMODE
	movlw	0x00
	movwf	temp1
	movwf	temp2
	movwf	temp3				; Variablen zurcksetzen!
	movlw	U_ERROR
	movwf	mode				; Mode setzen

userinput_ende:

	return

;############# Ende: UserInput ##########################################################



;########################################################################################
;
;	Modul:			WaitProcess
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Wartet eine Pause ab
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

WaitProcess

	btfsc	LAST_OUTPUT			; Abfrage: letzte Ausgabe?
	goto	wp_change_mode			; ja
	movlw	0x00				; nein
	xorwf	pause_anzeige,w
	btfss	STATUS,Z			; ist Zeit schon abgelaufen?
	goto	wait_output_ende		; nein, gehe zum Ende
	movlw	SEQUENCE_PROCESS		; ja, weiter ausgeben
	movwf	mode
	goto	wait_output_ende
wp_change_mode:
	movlw	0x00
	movwf	pause_anzeige			; Pause auf Null setzen
	movlw	USER_PROCESS
	movwf	mode				; Mode setzen
	bcf	LAST_OUTPUT
	movlw	U_INPUT_PAUSE
	movwf	pause_eingabe			; Timeout fr Usereingabe setzen
wait_output_ende:

	return

;############# Ende: WaitProcess ########################################################


;########################################################################################
;
;	Modul:			ModeSwitch
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Ruft entsprechend nach Mode die zugehrige Subroutine auf
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

ModeSwitch

	movlw	START_UP
	xorwf	mode,w
	btfsc	STATUS,Z
	call	Hochfahren			; Nur wenn Taster4 gedrckt, dann Selbsthaltung einschalten

	movlw	ANIMATION
	xorwf	mode,w
	btfsc	STATUS,Z
	call	SayHello			; Anfangsanimation

	movlw	CHOOSE_LEVEL
	xorwf	mode,w
	btfsc	STATUS,Z
	call	S_Grad_waehlen			; User whlt Schwierigkeitsgrad

	movlw	SEQUENCE_PROCESS
	xorwf	mode,w
	btfsc	STATUS,Z
	call	SequenceOutput			; Senso gibt aus

	movlw	WAIT_OUTPUT
	xorwf	mode,w
	btfsc	STATUS,Z
	call	WaitProcess			; Senso macht Pause bei Ausgabe

	movlw	USER_PROCESS
	xorwf	mode,w
	btfsc	STATUS,Z
	call	UserInput			; User gibt ein

	return

;############# Ende: ModeSwitch #########################################################


;########################################################################################
;
;	Modul:			LED_Ausgabe
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Wertet "merker_reg1" aus und setzt die LEDs entsprechend
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

LED_Ausgabe

	movlw	CHOOSE_LEVEL
	xorwf	mode,w				; Mode CHOOSE_LEVEL gesetzt?
	btfss	STATUS,Z
	goto	la_normal			; nein, anderer Mode gesetzt

						; Ansteuerung LEDs fr CHOOSE_LEVEL
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z			; Zeit vorbei?
	goto	la_end				; nein, gehe zum Ende

	movlw	BLINKZEIT_LED
	movwf	zeitdauer_led

	btfss	WAHL_LEVEL6
	goto	cl_pruef_led
	clrf	PORTB				; SONDERFALL!!!
	bcf	WAHL_LEVEL6

cl_pruef_led:
	btfsc	LED1_AKTIV			; LED1_AKTIV gesetzt?
	goto	cl_led1_toggle			; ja
	bcf	LED_1				; nein, LED1 ausschalten (vielleicht ja noch an)
	goto	cl_led2				; weiter zur Prfung LED2
cl_led1_toggle:
	btfss	LED_1				; LED1 an?
	goto	cl_led1_toggle_on		; nein --> anschalten
	bcf	LED_1				; ja --> ausschalten
	goto	cl_led2				; weiter gehts
cl_led1_toggle_on:
	bsf	LED_1				; LED1 anschalten

cl_led2:
	btfsc	LED2_AKTIV
	goto	cl_led2_toggle
	bcf	LED_2
	goto	cl_led3
cl_led2_toggle:
	btfss	LED_2
	goto	cl_led2_toggle_on
	bcf	LED_2
	goto	cl_led3
cl_led2_toggle_on:
	bsf	LED_2

cl_led3:
	btfsc	LED3_AKTIV
	goto	cl_led3_toggle
	bcf	LED_3
	goto	cl_led4
cl_led3_toggle:
	btfss	LED_3
	goto	cl_led3_toggle_on
	bcf	LED_3
	goto	cl_led4
cl_led3_toggle_on:
	bsf	LED_3

cl_led4:
	btfsc	LED4_AKTIV
	goto	cl_led4_toggle
	bcf	LED_4
	goto	la_end
cl_led4_toggle:
	btfss	LED_4
	goto	cl_led4_toggle_on
	bcf	LED_4
	goto	la_end
cl_led4_toggle_on:
	bsf	LED_4


	goto	la_end


la_normal:					; anderer Mode als CHOOSE_LEVEL gesetzt
	movlw	0x00
	xorwf	zeitdauer_led,w
	btfss	STATUS,Z			; Zeit vorbei?
	goto	la_normal_ein			; nein
	clrf	PORTB				; ja, PortB lschen
	goto	la_end

la_normal_ein:
	btfsc	LED1_AKTIV			; LED1_AKTIV gesetzt?
	goto	normal_led1_on			; ja
	goto	normal_led1_off			; nein, nchste LED prfen
normal_led1_on:
	bsf	LED_1				; LED1 setzen
	goto	normal_led2
normal_led1_off:
	bcf	LED_1				; LED1 lschen

normal_led2:
	btfsc	LED2_AKTIV			; LED2_AKTIV gesetzt?
	goto	normal_led2_on			; ja
	goto	normal_led2_off			; nein, nchste LED prfen
normal_led2_on:
	bsf	LED_2
	goto	normal_led3
normal_led2_off:
	bcf	LED_2

normal_led3:
	btfsc	LED3_AKTIV
	goto	normal_led3_on
	goto	normal_led3_off
normal_led3_on:
	bsf	LED_3
	goto	normal_led4
normal_led3_off:
	bcf	LED_3

normal_led4:
	btfsc	LED4_AKTIV
	goto	normal_led4_on
	goto	normal_led4_off
normal_led4_on:
	bsf	LED_4
	goto	la_end
normal_led4_off:
	bcf	LED_4
la_end:

	return

;############# Ende: LED_Ausgabe ########################################################



;########################################################################################
;
;	Modul:			ErfolgTon
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Gibt bei Erfolg einen Ton aus
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

ErfolgTon

	btfsc	PROC_AKTIV			; Wird der Ton gespielt, oder erster Aufruf?
	goto	et_pruefzeit			; Ton wird gespielt
	
	bsf	PROC_AKTIV			; erster Aufruf

	bsf	PIEZO_AKTIV			; Vorbereiten zur Ausgabe

	movlw	0x00
	xorwf	temp2,w
	btfss	STATUS,Z
	goto	et_ton2
	movlw	TON1_ERFOLG
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movwf	piezo_freq_l
	goto	et_set_ton_ende

et_ton2:
	movlw	0x01
	xorwf	temp2,w
	btfss	STATUS,Z
	goto	et_ton3
	movlw	TON2_ERFOLG
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movwf	piezo_freq_l
	goto	et_set_ton_ende

et_ton3:
	movlw	TON1_ERFOLG
	movwf	piezo_freq_h
	movwf	piezo_freq_akt
	movwf	piezo_freq_l

et_set_ton_ende:
	movlw	ERFOLG_TIME
	movwf	zeitdauer_piezo

	goto	erfolg_ton_ende

et_pruefzeit:
	movlw	0x00
	xorwf	zeitdauer_piezo,w
	btfss	STATUS,Z			; ist Zeit abgelaufen?
	goto	erfolg_ton_ende			; nein, gehe zum Ende
	bcf	PROC_AKTIV
	incf	temp2
	movlw	0x03
	xorwf	temp2,w
	btfss	STATUS,Z
	goto	erfolg_ton_ende
	movlw	0x00
	movwf	temp2
	movwf	temp3				; Variable zurcksetzen
	bcf	PROC_AKTIV			; ja, ndere Mode
	movlw	ANIMATION
	movwf	mode				; Reset

erfolg_ton_ende:
	return

;############# Ende: ErfolgTon ##########################################################


;########################################################################################
;
;	Modul:			FehlerTon
;
;	Version:		1.0
;
;	Bearbeiter: 		F. Schubert
;
;	Datum:			14/10/2002
;
;	Funktion:		Gibt bei einer falschen Eingabe des Users einen Ton aus
;
;	Controller:		PIC16F876
;
;	Eingabeparameter:	
;
;	Ausgabeparameter:	
;
;	Vernderte register:
;
;	Vernderte flags:
;
;	Aufgerufene module:
;
;	Stacktiefe:
;
;	Registerbank:
;
;	Speicherseite:
;
;	Hinweise:		
;
;########################################################################################

FehlerTon

	btfsc	PROC_AKTIV			; Wird der Ton gespielt, oder erster Aufruf?
	goto	ft_pruefzeit			; Ton wird gespielt
	
	bsf	PROC_AKTIV			; erster Aufruf

	bsf	PIEZO_AKTIV			; Vorbereiten zur Ausgabe
	movlw	A_TON_FEHLER
	movwf	piezo_freq_h
	movwf	piezo_freq_akt

	movlw	E_TON_FEHLER
	movwf	piezo_freq_l

	movlw	FEHLER_TIME
	movwf	zeitdauer_piezo

	goto	fehler_ton_ende

ft_pruefzeit:
	movlw	0x00
	xorwf	zeitdauer_piezo,w
	btfss	STATUS,Z			; ist Zeit abgelaufen?
	goto	fehler_ton_ende			; nein, gehe zum Ende
	bcf	PROC_AKTIV			; ja, ndere Mode
	movlw	ANIMATION
	movwf	mode				; Reset

fehler_ton_ende:
	return

;############# Ende: FehlerTon ##########################################################



;########################################################################################
;########################################################################################
;	Hauptprogramm
;########################################################################################
;########################################################################################

start	nop
	call	ConfigController		; Controller initialisieren
	call	ClearRAM			; Lschen des gesamten RAMs

	movlw 	AUSSCHALTZEIT
	movwf	ausschalten			; setzen der Ausschaltzeit

	movlw	START_UP
	movwf	mode				; ersten Mode setzen

loop:	
	;============= Eingabe =============
	call	Taster_lesen			; ist eine Taste gedrckt (mit Entprellroutine)?
	;========== Ende Eingabe ===========

	;========== Verarbeitung ===========
	call	TimeBase			; herstellen der Zeitbasen
	call	ModeSwitch			; Verzweigung der Zustnde
	;======= Ende Verarbeitung =========

	;============= Ausgabe =============
	call	LED_Ausgabe			; tja, der Name sagt schon alles...

	movlw	U_ERROR
	xorwf	mode,w
	btfsc	STATUS,Z
	call	FehlerTon

	movlw	U_SUCCESS
	xorwf	mode,w
	btfsc	STATUS,Z
	call	ErfolgTon
	;========== Ende Ausgabe ===========
	goto	loop


	end
